﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = ChartData.GenerateData(20);
        }

        private static IEnumerable<DependencyObject> VisualDescendants(DependencyObject obj)
        {
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(obj); i++)
            {
                var child = VisualTreeHelper.GetChild(obj, i);
                yield return child;
                foreach (var subChild in VisualDescendants(child))
                {
                    yield return subChild;
                }
            }
        }
        public static void SelectClosest(Series series, Point point)
        {
            double minDist = double.PositiveInfinity;
            TrackingGrid closest = null;
            FrameworkElement closestContent = null;
            FrameworkElement beforeVisible = null;

            foreach (var grid in TrackingGrid.Items()
                .Where((i) => i.Series == series))
            {
                double left = GetLeft(series, grid.Item);
                double dist = System.Math.Abs(point.X - left);
                var content = grid.VisibilityItem;

                if (content != null &&
                    content.Visibility == Visibility.Visible)
                {
                    beforeVisible = content;
                    content.Visibility
                        = Visibility.Collapsed;
                }
                if (dist < minDist)
                {
                    minDist = dist;
                    closest = grid.Item;
                    closestContent = content;
                }
            }

            if (closest != null)
            {
                if (closestContent != null)
                {
                    closestContent.Visibility
                        = Visibility.Visible;
                }
            }
            else
            {
                if (beforeVisible != null)
                {
                    beforeVisible.Visibility
                        = Visibility.Visible;
                }
            }
        }
        private static Marker FindMarker(FrameworkElement item)
        {
            while (item != null)
            {
                if (item is Marker)
                {
                    return item as Marker;
                }
                item = VisualTreeHelper.GetParent(item)
                    as FrameworkElement;
            }
            return null;
        }
        private static double GetLeft(Series series, FrameworkElement item)
        {
            Marker m = FindMarker(item);

            if (m == null)
            {
                return double.NaN;
            }
            if (m.Visibility == Visibility.Collapsed)
            {
                return double.NaN;
            }
            TranslateTransform t = m.RenderTransform as TranslateTransform;
            if (t == null)
            {
                return double.NaN;
            }
            return t.X + m.ActualWidth / 2.0;
        }

        private void OnSeriesCursorMouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            XamDataChart chart = sender as XamDataChart;
            if (chart == null)
            {
                return;
            }

            foreach (var series in chart.Series)
            {
                var seriesPos = e.GetPosition(series);
                if (seriesPos.X >= 0 &&
                    seriesPos.X < series.ActualWidth &&
                    seriesPos.Y >= 0 &&
                    seriesPos.Y < series.ActualHeight)
                {
                    SelectClosest(
                    series, seriesPos);
                }
            }
        }

        private void xamDataChart1_Loaded(object sender, RoutedEventArgs e)
        {

        }
    }

    public class ChartData
    {
        public DateTime Date { get; set; }
        public double Value1 { get; set; }
        public double Value2 { get; set; }
        public double Value3 { get; set; }

        public static ObservableCollection<ChartData> GenerateData(int count)
        {
            ObservableCollection<ChartData> data = new ObservableCollection<ChartData>();

            Random r = new Random();

            ChartData data0 = new ChartData
            {
                Date = DateTime.Now,
                Value1 = r.Next(20, 40),
                Value2 = r.Next(20, 40),
                Value3 = r.Next(20, 40),
            };

            data.Add(data0);

            for (int i = 1; i < count; i++)
            {
                ChartData dat = new ChartData
                {
                    Date = DateTime.Now.AddDays(i),
                };

                if (r.NextDouble() > .5)
                {
                    dat.Value1 = data[i - 1].Value1 + r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 + r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 + r.NextDouble();
                }
                else
                {
                    dat.Value1 = data[i - 1].Value1 - r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 - r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 - r.NextDouble();
                }

                data.Add(dat);
            }

            return data;
        }
    }

    public class TrackingGrid : Grid
    {
        public static readonly DependencyProperty SeriesProperty =
           DependencyProperty.Register(
           "Series",
           typeof(Control),
           typeof(TrackingGrid),
           new PropertyMetadata(null,
               (o, e) =>
               {
                   (o as TrackingGrid)
                       .OnSeriesChanged(e);
               }));

        private void OnSeriesChanged(
            DependencyPropertyChangedEventArgs e)
        {
            Refresh();
        }

        public Control Series
        {
            get { return (Control)GetValue(SeriesProperty); }
            set { SetValue(SeriesProperty, value); }
        }

        public static readonly DependencyProperty VisibilityItemProperty =
           DependencyProperty.Register(
           "VisibilityItem",
           typeof(VisibilityItem),
           typeof(TrackingGrid),
           new PropertyMetadata(null,
               (o, e) =>
               {
                   (o as TrackingGrid)
                       .OnVisibilityItemChanged(e);
               }
               ));

        private void OnVisibilityItemChanged(
            DependencyPropertyChangedEventArgs e)
        {
            Refresh();
        }

        public VisibilityItem VisibilityItem
        {
            get { return (VisibilityItem)GetValue(VisibilityItemProperty); }
            set { SetValue(VisibilityItemProperty, value); }
        }

        public TrackingGrid()
        {
            this.Loaded += TrackingGrid_Loaded;
            this.Unloaded += TrackingGrid_Unloaded;
        }

        private void Refresh()
        {
            if (_items.ContainsKey(this))
            {
                _items.Remove(this);
            }
            _items.Add(this, new ItemInfo
            {
                Series = Series as Series,
                Item = this,
                VisibilityItem = VisibilityItem
            });
        }

        void TrackingGrid_Unloaded(object sender, RoutedEventArgs e)
        {
            _items.Remove(this);
        }

        void TrackingGrid_Loaded(object sender, RoutedEventArgs e)
        {
            Refresh();
        }

        public class ItemInfo
        {
            public Series Series { get; set; }
            public TrackingGrid Item { get; set; }
            public VisibilityItem VisibilityItem { get; set; }
        }

        private static Dictionary<TrackingGrid, ItemInfo> _items
            = new Dictionary<TrackingGrid, ItemInfo>();

        public static IEnumerable<ItemInfo> Items()
        {
            return _items.Values;
        }
    }

    public class VisibilityItem : ContentControl
    {
    }
}
